/* EOAccessAdditions.h created by lindberg on Mon 20-Dec-1999 */

#import <EOAccess/EOModel.h>
#import <EOAccess/EOEntity.h>
#import <EOAccess/EORelationship.h>
#import "SystemType.h"

/* EOAccess extension methods for use by templates */

@interface EOModel (EOGenerator)

// Tries to find the framework name. First looks to see if a value for
// "EOGeneratorFrameworkName" exists in the model's -userInfo dictionary, and if
// not, looks at the model's -path to find the framework the .eomodeld is in.
// Both PB.project files and path components ending in ".framework" are looked
// for, so it should work for .eomodels insided installed frameworks or in a
// development tree. If the framework name could not be found, returns nil.
- (NSString *)frameworkName;

// Uses isLike: to do the comparisons
- (NSArray *)entityNamesMatchingWildcard:(NSString *)wildcard;

// Returns back the list of entities that are defined as concrete.  IE: those
// whose class is not EOGenericRecord
- (NSArray *)concreteEntityNames;
- (NSArray *)concreteEntities;

// Returns back the list of entities that are defined as generic.  IE: those
// whose class is EOGenericRecord
- (NSArray *)genericEntityNames;
- (NSArray *)genericEntities;

@end

@interface EOEntity (EOGenerator)

- (BOOL)hasParentEntity;
- (NSString *)frameworkName;

- (NSString *)objcImportString;
- (NSString *)objcImportStringInRelationToEntity:(EOEntity *)sourceEntity;
- (NSString *)parentObjCImportString;

//uniqued array of relationships' -objcImportString
- (NSArray *)referencedObjCImportStrings;
- (NSArray *)arrayWithParentObjCImportStringIfNeeded;

- (NSDictionary *)storedProcedures;
- (EOStoredProcedure *)storedProcedureForFetchAll;
- (EOStoredProcedure *)storedProcedureForFetchWithPrimaryKey;
- (EOStoredProcedure *)storedProcedureForInsert;
- (EOStoredProcedure *)storedProcedureForDelete;
- (EOStoredProcedure *)storedProcedureForPrimaryKeyGeneration;

#ifndef EOF2_ONLY
// JavaClient methods... reimplementation of ones in EOJavaClientExtensions.EOMBundle
- (NSArray *)clientClassAttributes;
- (NSArray *)clientClassToOneRelationships;
- (NSArray *)clientClassToManyRelationships;
- (NSArray *)clientClassScalarAttributes;
- (NSArray *)clientClassNonScalarAttributes;
#endif

#ifdef EOF2_ONLY
// Define for use by EOF2 systems
- (NSString *)classNameWithoutPackage;
- (NSArray *)fetchSpecificationNames;
- (EOFetchSpecification *)fetchSpecificationNamed:(NSString *)name;
- (NSArray *)clientClassProperties;
#endif

@end

@interface EORelationship (EOGenerator)

// If the destination entity is in an external framework, returns
// <Framework/ClassName.h>, otherwise returns "ClassName.h".
- (NSString *)objcImportString;
- (BOOL)isClassProperty;
- (BOOL)isClientClassProperty;

@end

@interface EOAttribute (EOGenerator)

- (BOOL)isPrimaryKey;
- (BOOL)isClassProperty;
- (BOOL)isClientClassProperty;
- (BOOL)isSnapshotProperty;

- (NSString *)fullExternalType;

#ifdef EOF2_ONLY
// Define for use by EOF2 systems
- (id)prototype;
#endif

@end
