//
//  _MiscMergeIfCommand.m
//            Written by Don Yacktman and Carl Lindberg
//        Copyright 2000 by Don Yacktman and Carl Lindberg.
//                     All rights reserved.
//      This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "_MiscMergeIfCommand.h"
#import <Foundation/NSUtilities.h>
#import <Foundation/NSValue.h>
#import <Foundation/NSScanner.h>
#import "MiscMergeCommandBlock.h"
#import "MiscMergeCondition.h"

@implementation _MiscMergeIfCommand

- (id)init
{
	[super init];
	trueBlock = [[MiscMergeCommandBlock alloc] initWithOwner:self];
	return self;
}

- (void)dealloc
{
	[trueBlock release];
	[elseBlock release];
    [condition release];
	[super dealloc];
}

- (BOOL)parseFromScanner:(NSScanner *)aScanner template:(MiscMergeTemplate *)template
{
	[self eatKeyWord:@"if" fromScanner:aScanner isOptional:NO];
    condition = [[self getConditionFromScanner:aScanner] retain];
	[template pushCommandBlock:trueBlock];
	return YES;
}

- (void)handleElseInTemplate:(MiscMergeTemplate *)template
{
	if (elseBlock == nil) elseBlock = [[MiscMergeCommandBlock alloc] initWithOwner:self];
	[template popCommandBlock:trueBlock];
	[template pushCommandBlock:elseBlock];
}

- (void)handleEndifInTemplate:(MiscMergeTemplate *)template
{
	[template popCommandBlock];
}

- (void)executeForMerge:(MiscMergeEngine *)aMerger
{
	if ([self evaluateConditionInMerger:aMerger])
		[aMerger executeCommandBlock:trueBlock];
	else
		if (elseBlock) [aMerger executeCommandBlock:elseBlock];
}


- (BOOL)evaluateConditionInMerger:(MiscMergeEngine *)anEngine
{
    return [condition evaluateWithEngine:anEngine];
}

@end

